@@ -182,7 +182,7 @@ class FileBase(object):
 
     @staticmethod
     def _mode_to_flags(mode):
-        flags = os.O_RDONLY | os.O_NONBLOCK # always non-blocking
+        flags = os.O_RDONLY | os.O_NONBLOCK  # always non-blocking
         if (('w' in mode or 'a' in mode) and 'r' in mode) or '+' in mode:
             # both read and write
             flags |= os.O_RDWR
@@ -194,6 +194,10 @@ class FileBase(object):
             # append-write mode
             flags |= os.O_APPEND
 
+        # builtin open() also creates the file on writable opens
+        if flags & (os.O_WRONLY|os.O_RDWR):
+            flags |= os.O_CREAT
+
         return flags
 
 
@@ -214,10 +218,6 @@ class File(FileBase):
         # translate mode into the proper open flags
         flags = self._mode_to_flags(mode)
 
-        # if write or append mode and the file doesn't exist, create it
-        if flags & (os.O_WRONLY | os.O_RDWR) and not os.path.exists(name):
-            _open(name, 'w').close()
-
         # open the file, get a descriptor
         try:
             self._fileno = _osopen(name, flags)
@@ -320,7 +320,7 @@ class File(FileBase):
 
         :returns: a new :class:`File` object connected to the descriptor
         """
-        fp = object.__new__(cls) # bypass __init__
+        fp = object.__new__(cls)  # bypass __init__
         fp.mode = mode
         fp._fileno = fd
         fp._rbuf = StringIO()
